@@ -7,19 +7,21 @@ import hashlib
 import re
 import os
 import fnmatch
-import time
 import email.utils
 import email.policy
 import requests
 import urllib.parse
+import datetime
+import time
 
+from pathlib import Path
 from tempfile import mkstemp
 
 from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-VERSION = '0.3.1'
+__VERSION__ = '0.3.4'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -75,13 +77,17 @@ DEFAULT_CONFIG = {
     # strict: must match one of the uids on the key to pass
     # loose: any valid and trusted key will be accepted
     'attestation-uid-match': 'loose',
+    # How many days before we consider attestation too old?
+    'attestation-staleness-days': '30',
     # NB! This whole behaviour will change once public-inbox
     # gains support for cross-list searches
-    'attestation-query-url': LOREADDR + '/signatures',
+    'attestation-query-url': LOREADDR + '/signatures/',
     # We'll use the default gnupg homedir, unless you set it here
     'attestation-gnupghome': None,
     # Do you like simple or fancy checkmarks?
     'attestation-checkmarks': 'fancy',
+    # How long to keep things in cache before expiring (minutes)?
+    'cache-expire': '10',
     # If this is not set, we'll use what we find in 
     # git-config for gpg.program, and if that's not set,
     # we'll use "gpg" and hope for the better
@@ -99,12 +105,15 @@ ATTESTATIONS = list()
 SUBKEY_DATA = dict()
 # Used for storing our requests session
 REQSESSION = None
+# Indicates that we've cleaned cache already
+_CACHE_CLEANED = False
 
 
 class LoreMailbox:
     def __init__(self):
         self.msgid_map = dict()
         self.series = dict()
+        self.covers = dict()
         self.followups = list()
         self.unknowns = list()
 
@@ -147,8 +156,11 @@ class LoreMailbox:
             logger.critical('All patches in series v%s are missing.', lser.revision)
             return None
 
-        # Do we have a cover letter for it?
-        if not lser.has_cover:
+        # Grab our cover letter if we have one
+        if revision in self.covers.keys():
+            lser.add_patch(self.covers[revision])
+            lser.has_cover = True
+        else:
             # Let's find the first patch with an in-reply-to and see if that
             # is our cover letter
             for member in lser.patches:
@@ -189,12 +201,12 @@ class LoreMailbox:
             else:
                 pmsg = self.msgid_map[fmsg.in_reply_to]
 
-            trailers = fmsg.trailers
+            trailers = set(fmsg.trailers)
             lvl = 1
             while True:
                 logger.debug('%sParent: %s', ' ' * lvl, pmsg.full_subject)
                 logger.debug('%sTrailers:', ' ' * lvl)
-                for trailer in set(trailers):
+                for trailer in trailers:
                     logger.debug('%s%s: %s', ' ' * (lvl+1), trailer[0], trailer[1])
                 found = False
                 if lser.revision != pmsg.revision:
@@ -202,14 +214,14 @@ class LoreMailbox:
                 for lmsg in lser.patches:
                     if lmsg is not None and lmsg.msgid == pmsg.msgid:
                         # Confirmed, this is our parent patch
-                        lmsg.followup_trailers += trailers
+                        lmsg.followup_trailers.update(trailers)
                         found = True
                         break
                 if found:
                     break
                 elif pmsg.in_reply_to and pmsg.in_reply_to in self.msgid_map:
                     lvl += 1
-                    trailers += pmsg.trailers
+                    trailers.update(pmsg.trailers)
                     pmsg = self.msgid_map[pmsg.in_reply_to]
                 else:
                     break
@@ -221,42 +233,58 @@ class LoreMailbox:
         logger.debug('Looking at: %s', lmsg.full_subject)
         self.msgid_map[lmsg.msgid] = lmsg
 
-        if lmsg.has_diff or lmsg.has_diffstat:
+        if lmsg.counter == 0 and lmsg.has_diffstat:
+            # Cover letter
+            # Add it to covers -- we'll deal with them later
+            logger.debug('  adding as v%s cover letter', lmsg.revision)
+            self.covers[lmsg.revision] = lmsg
+            return
+
+        if lmsg.reply:
+            # We'll figure out where this belongs later
+            logger.debug('  adding to followups')
+            self.followups.append(lmsg)
+            return
+
+        if re.search(r'^Comment: att-fmt-ver:', lmsg.body, re.I | re.M):
+            logger.debug('Found attestation message')
+            LoreAttestationDocument.load_from_string(lmsg.msgid, lmsg.body)
+            # We don't keep it, because it's not useful for us beyond this point
+            return
+
+        if lmsg.has_diff:
+            if lmsg.revision not in self.series:
+                if lmsg.revision_inferred and lmsg.in_reply_to:
+                    # We have an inferred revision here.
+                    # Do we have an upthread cover letter that specifies a revision?
+                    irt = self.get_by_msgid(lmsg.in_reply_to)
+                    if irt is not None and irt.has_diffstat and not irt.has_diff:
+                        # Yes, this is very likely our cover letter
+                        logger.debug('  fixed revision to v%s', irt.revision)
+                        lmsg.revision = irt.revision
+
+            # Run our check again
             if lmsg.revision not in self.series:
                 self.series[lmsg.revision] = LoreSeries(lmsg.revision, lmsg.expected)
                 if len(self.series) > 1:
                     logger.info('Found new series v%s', lmsg.revision)
-            if lmsg.has_diff:
-                # Attempt to auto-number series from the same author who did not bother
-                # to set v2, v3, etc in the patch revision
-                if (lmsg.counter == 1 and lmsg.counters_inferred
-                        and not lmsg.reply and lmsg.lsubject.patch and not lmsg.lsubject.resend):
-                    omsg = self.series[lmsg.revision].patches[lmsg.counter]
-                    if (omsg is not None and omsg.counters_inferred and lmsg.fromemail == omsg.fromemail
-                            and omsg.date < lmsg.date):
-                        lmsg.revision = len(self.series) + 1
-                        self.series[lmsg.revision] = LoreSeries(lmsg.revision, lmsg.expected)
-                        logger.info('Assuming new revision: v%s (%s)', lmsg.revision, lmsg.full_subject)
-                logger.debug('  adding as patch')
-                self.series[lmsg.revision].add_patch(lmsg)
-            elif lmsg.counter == 0 and lmsg.has_diffstat:
-                # Bona-fide cover letter
-                logger.debug('  adding as cover letter')
-                self.series[lmsg.revision].add_cover(lmsg)
-            elif lmsg.reply:
-                # We'll figure out where this belongs later
-                logger.debug('  adding to followups')
-                self.followups.append(lmsg)
-        elif lmsg.reply:
-            logger.debug('  adding to followups')
-            self.followups.append(lmsg)
-        elif re.search(r'^Comment: att-fmt-ver:', lmsg.body, re.I | re.M):
-            logger.debug('Found attestation message')
-            LoreAttestationDocument.load_from_string(lmsg.msgid, lmsg.body)
-            # We don't keep it, because it's not useful for us beyond this point
-        else:
-            logger.debug('  adding to unknowns')
-            self.unknowns.append(lmsg)
+
+            # Attempt to auto-number series from the same author who did not bother
+            # to set v2, v3, etc in the patch revision
+            if (lmsg.counter == 1 and lmsg.counters_inferred
+                    and not lmsg.reply and lmsg.lsubject.patch and not lmsg.lsubject.resend):
+                omsg = self.series[lmsg.revision].patches[lmsg.counter]
+                if (omsg is not None and omsg.counters_inferred and lmsg.fromemail == omsg.fromemail
+                        and omsg.date < lmsg.date):
+                    lmsg.revision = len(self.series) + 1
+                    self.series[lmsg.revision] = LoreSeries(lmsg.revision, lmsg.expected)
+                    logger.info('Assuming new revision: v%s (%s)', lmsg.revision, lmsg.full_subject)
+            logger.debug('  adding as patch')
+            self.series[lmsg.revision].add_patch(lmsg)
+            return
+
+        logger.debug('  adding to unknowns')
+        self.unknowns.append(lmsg)
 
 
 class LoreSeries:
@@ -309,11 +337,7 @@ class LoreSeries:
             self.patches[lmsg.counter] = lmsg
         self.complete = not (None in self.patches[1:])
 
-    def add_cover(self, lmsg):
-        self.add_patch(lmsg)
-        self.has_cover = True
-
-    def get_slug(self):
+    def get_slug(self, extended=False):
         # Find the first non-None entry
         lmsg = None
         for lmsg in self.patches:
@@ -323,10 +347,16 @@ class LoreSeries:
         if lmsg is None:
             return 'undefined'
 
-        prefix = time.strftime('%Y%m%d', lmsg.date[:9])
+        prefix = lmsg.date.strftime('%Y%m%d')
         authorline = email.utils.getaddresses(lmsg.msg.get_all('from', []))[0]
-        author = re.sub(r'\W+', '_', authorline[1]).strip('_').lower()
-        slug = '%s_%s' % (prefix, author)
+        if extended:
+            local = authorline[1].split('@')[0]
+            unsafe = '%s_%s_%s' % (prefix, local, lmsg.subject)
+            slug = re.sub(r'\W+', '_', unsafe).strip('_').lower()
+        else:
+            author = re.sub(r'\W+', '_', authorline[1]).strip('_').lower()
+            slug = '%s_%s' % (prefix, author)
+
         if self.revision != 1:
             slug = 'v%s_%s' % (self.revision, slug)
 
@@ -345,6 +375,10 @@ class LoreSeries:
 
         attdata = [None] * self.expected
         attpolicy = config['attestation-policy']
+        try:
+            attstaled = int(config['attestation-staleness-days'])
+        except ValueError:
+            attstaled = 30
         exact_from_match = False
         if config['attestation-uid-match'] == 'strict':
             exact_from_match = True
@@ -357,14 +391,15 @@ class LoreSeries:
             attfail = FAIL_SIMPLE
 
         at = 1
+        atterrors = list()
         for lmsg in self.patches[1:]:
             if lmsg is not None:
                 if self.has_cover and covertrailers and self.patches[0].followup_trailers:
-                    lmsg.followup_trailers += self.patches[0].followup_trailers
+                    lmsg.followup_trailers.update(self.patches[0].followup_trailers)
                 if addmysob:
-                    lmsg.followup_trailers.append(('Signed-off-by', '%s <%s>' % (usercfg['name'], usercfg['email'])))
+                    lmsg.followup_trailers.add(('Signed-off-by', '%s <%s>' % (usercfg['name'], usercfg['email'])))
                 if addlink:
-                    lmsg.followup_trailers.append(('Link', linkmask % lmsg.msgid))
+                    lmsg.followup_trailers.add(('Link', linkmask % lmsg.msgid))
 
                 if attpolicy != 'off':
                     lore_lookup = False
@@ -375,11 +410,35 @@ class LoreSeries:
                     if attdoc is None:
                         if attpolicy in ('softfail', 'hardfail'):
                             logger.info('  %s %s', attfail, lmsg.full_subject)
+                            # Which part failed?
+                            failed = ['commit metadata', 'commit message', 'patch content']
+                            for attdoc in ATTESTATIONS:
+                                for i, m, p in attdoc.hashes:
+                                    if p == lmsg.attestation.p:
+                                        failed.remove('patch content')
+                                    if m == lmsg.attestation.m:
+                                        failed.remove('commit message')
+                                    if i == lmsg.attestation.i:
+                                        failed.remove('commit metadata')
+                            atterrors.append('Patch %s/%s failed attestation (%s)' % (at, lmsg.expected,
+                                                                                      ', '.join(failed)))
                         else:
                             logger.info('  %s', lmsg.full_subject)
                     else:
-                        logger.info('  %s %s', attpass, lmsg.full_subject)
-                        attdata[at-1] = attdoc.attestor.get_trailer(lmsg.fromemail)
+                        if attpolicy == 'check':
+                            # switch to softfail policy now that we have at least one hit
+                            attpolicy = 'softfail'
+                        # Make sure it's not too old compared to the message date
+                        # Timezone doesn't matter as we calculate whole days
+                        tdelta = lmsg.date.replace(tzinfo=None) - attdoc.sigdate
+                        if tdelta.days > attstaled:
+                            # Uh-oh, attestation is too old!
+                            logger.info('  %s %s', attfail, lmsg.full_subject)
+                            atterrors.append('Attestation for %s/%s is over %sd old: %sd' % (at, lmsg.expected,
+                                                                                             attstaled, tdelta.days))
+                        else:
+                            logger.info('  %s %s', attpass, lmsg.full_subject)
+                            attdata[at-1] = attdoc.attestor.get_trailer(lmsg.fromemail)
                 else:
                     logger.info('  %s', lmsg.full_subject)
 
@@ -402,7 +461,7 @@ class LoreSeries:
                 logger.info('  %s %s', attpass, trailer)
             return mbx
 
-        errors = set()
+        errors = set(atterrors)
         for attdoc in ATTESTATIONS:
             errors.update(attdoc.errors)
 
@@ -410,7 +469,7 @@ class LoreSeries:
             logger.critical('  ---')
             logger.critical('  Attestation is available, but did not succeed:')
             for error in errors:
-                logger.critical('  %s %s', attfail, error)
+                logger.critical('    %s %s', attfail, error)
 
         if attpolicy == 'hardfail':
             import sys
@@ -451,8 +510,8 @@ class LoreMessage:
         self.body = None
         self.has_diff = False
         self.has_diffstat = False
-        self.trailers = list()
-        self.followup_trailers = list()
+        self.trailers = set()
+        self.followup_trailers = set()
 
         self.attestation = None
 
@@ -481,7 +540,7 @@ class LoreMessage:
         except IndexError:
             pass
 
-        self.date = email.utils.parsedate_tz(str(self.msg['Date']))
+        self.date = email.utils.parsedate_to_datetime(str(self.msg['Date']))
 
         diffre = re.compile(r'^(---.*\n\+\+\+|GIT binary patch)', re.M | re.I)
         diffstatre = re.compile(r'^\s*\d+ file.*\d+ (insertion|deletion)', re.M | re.I)
@@ -521,7 +580,7 @@ class LoreMessage:
             matches = re.findall(r'^\s*Fixes:[ \t]+([a-f0-9]+\s+\(.*\))\s*$', self.body, re.MULTILINE)
             if matches:
                 for tvalue in matches:
-                    self.trailers.append(('Fixes', tvalue))
+                    self.trailers.add(('Fixes', tvalue))
 
             # Do we have something that looks like a person-trailer?
             matches = re.findall(r'^\s*([\w-]+):[ \t]+(.*<\S+>)\s*$', self.body, re.MULTILINE)
@@ -563,7 +622,7 @@ class LoreMessage:
                             logger.debug('  trailer fuzzy name match')
                             tmatch = True
                     if tmatch:
-                        self.trailers.append((tname, tvalue))
+                        self.trailers.add((tname, tvalue))
                     else:
                         logger.debug('  ignoring "%s: %s" due to from mismatch (from: %s %s)', tname, tvalue,
                                      self.fromname, self.fromemail)
@@ -623,7 +682,6 @@ class LoreMessage:
         # XXX: This currently doesn't work for git binary patches
         #
         diff = diff.replace('\r', '')
-        diff = diff.strip() + '\n'
 
         # For keeping a buffer of lines preceding @@ ... @@
         buflines = list()
@@ -654,8 +712,9 @@ class LoreMessage:
             if pp > 0:
                 # Inside the patch
                 phasher.update((line + '\n').encode('utf-8'))
-                if line[0] != '-':
-                    pp -= 1
+                if len(line) and line[0] == '-':
+                    continue
+                pp -= 1
                 continue
             # Not anything we recognize, so stick into buflines
             buflines.append(line)
@@ -724,7 +783,8 @@ class LoreMessage:
 
                 # Now we add mix-in trailers
                 btrailers.reverse()
-                trailers = btrailers + self.followup_trailers
+                trailers = set(self.followup_trailers)
+                trailers.update(set(btrailers))
                 added = list()
                 if trailer_order is None:
                     trailer_order = DEFAULT_TRAILER_ORDER
@@ -735,9 +795,9 @@ class LoreMessage:
                         if fnmatch.fnmatch(trailer[0].lower(), trailermatch.strip()):
                             fixlines.append('%s: %s' % trailer)
                             if trailer not in btrailers:
-                                logger.info('    Added: %s: %s' % trailer)
+                                logger.info('    + %s: %s' % trailer)
                             else:
-                                logger.debug('     Kept: %s: %s' % trailer)
+                                logger.debug('    . %s: %s' % trailer)
                             added.append(trailer)
                 trailersdone = True
             fixlines.append(line)
@@ -820,8 +880,8 @@ class LoreSubject:
         # Remove any brackets inside brackets
         while True:
             oldsubj = subject
-            subject = re.sub(r'^\s*\[([^\]]*)\[([^\]]*)\]', '[\\1\\2]', subject)
-            subject = re.sub(r'^\s*\[([^\]]*)\]([^\]]*)\]', '[\\1\\2]', subject)
+            subject = re.sub(r'^\s*\[([^\]]*)\[([^\[\]]*)\]', '[\\1\\2]', subject)
+            subject = re.sub(r'^\s*\[([^\]]*)\]([^\[\]]*)\]', '[\\1\\2]', subject)
             if oldsubj == subject:
                 break
 
@@ -935,6 +995,7 @@ class LoreAttestationDocument:
         self.good = False
         self.valid = False
         self.trusted = False
+        self.sigdate = None
         self.passing = False
         self.attestor = None
         self.hashes = set()
@@ -956,16 +1017,20 @@ class LoreAttestationDocument:
                 keyid = gs_matches.groups()[0]
                 self.attestor = LoreAttestor(keyid)
                 puid = '%s <%s>' % self.attestor.get_primary_uid()
-                if re.search(r'^\[GNUPG:\] VALIDSIG', output, re.M):
+                vs_matches = re.search(r'^\[GNUPG:\] VALIDSIG ([0-9A-F]+) (\d{4}-\d{2}-\d{2}) (\d+)', output, re.M)
+                if vs_matches:
                     logger.debug('  VALIDSIG')
                     self.valid = True
+                    ymd = vs_matches.groups()[1]
+                    self.sigdate = datetime.datetime.strptime(ymd, '%Y-%m-%d')
                     # Do we have a TRUST_(FULLY|ULTIMATE)?
-                    matches = re.search(r'^\[GNUPG:\] TRUST_(FULLY|ULTIMATE)', output, re.M)
-                    if matches:
-                        logger.debug('  TRUST_%s', matches.groups()[0])
+                    ts_matches = re.search(r'^\[GNUPG:\] TRUST_(FULLY|ULTIMATE)', output, re.M)
+                    if ts_matches:
+                        logger.debug('  TRUST_%s', ts_matches.groups()[0])
                         self.trusted = True
                     else:
-                        self.errors.add('Insufficient trust on key: %s (%s)' % (keyid, puid))
+                        self.errors.add('Insufficient trust (model=%s): %s (%s)'
+                                        % (config['attestation-trust-model'], keyid, puid))
                 else:
                     self.errors.add('Signature not valid from key: %s (%s)' % (keyid, puid))
         else:
@@ -980,6 +1045,15 @@ class LoreAttestationDocument:
         if self.good and self.valid and self.trusted:
             self.passing = True
 
+        if source.find('http') == 0:
+            # We only cache known-good attestations obtained from remote
+            cachedir = get_cache_dir()
+            cachename = '%s.attestation' % urllib.parse.quote_plus(source.strip('/').split('/')[-1])
+            fullpath = os.path.join(cachedir, cachename)
+            with open(fullpath, 'w') as fh:
+                logger.debug('Saved attestation in cache: %s', cachename)
+                fh.write(sigdata)
+
         hg = [None, None, None]
         for line in sigdata.split('\n'):
             # It's a yaml document, but we don't parse it as yaml for safety reasons
@@ -1016,26 +1090,60 @@ class LoreAttestationDocument:
             out.append('  | %s-%s-%s' % (hg[0][:8], hg[1][:8], hg[2][:8]))
         return '\n'.join(out)
 
+    @staticmethod
+    def get_from_cache(attid):
+        cachedir = get_cache_dir()
+        attdocs = list()
+        for entry in os.listdir(cachedir):
+            if entry.find('.attestation') <= 0:
+                continue
+            fullpath = os.path.join(cachedir, entry)
+            with open(fullpath, 'r') as fh:
+                content = fh.read()
+                # Can't be 0, because it has to have pgp ascii wrapper
+                if content.find(attid) > 0:
+                    attdoc = LoreAttestationDocument(fullpath, content)
+                    attdocs.append(attdoc)
+        return attdocs
+
     @staticmethod
     def get_from_lore(attid):
+        attdocs = list()
         # XXX: Querying this via the Atom feed is a temporary kludge until we have
         #      proper search API on lore.kernel.org
+        cachedir = get_cache_dir()
+        cachefile = os.path.join(cachedir, '%s.lookup' % urllib.parse.quote_plus(attid))
+        status = None
+        if os.path.exists(cachefile):
+            with open(cachefile, 'r') as fh:
+                try:
+                    status = int(fh.read())
+                except ValueError:
+                    pass
+        if status is not None and status != 200:
+            logger.debug('Cache says looking up %s = %s', attid, status)
+            return attdocs
+
         config = get_main_config()
         queryurl = '%s?%s' % (config['attestation-query-url'],
                               urllib.parse.urlencode({'q': attid, 'x': 'A', 'o': '-1'}))
         logger.debug('Query URL: %s', queryurl)
         session = get_requests_session()
         resp = session.get(queryurl)
-        content = resp.content.decode('utf-8')
+        if resp.status_code != 200:
+            # Record this as a bad hit
+            with open(cachefile, 'w') as fh:
+                fh.write(str(resp.status_code))
+
         matches = re.findall(
             r'link\s+href="([^"]+)".*?(-----BEGIN PGP SIGNED MESSAGE-----.*?-----END PGP SIGNATURE-----)',
-            content, flags=re.DOTALL
+            resp.content.decode('utf-8'), flags=re.DOTALL
         )
 
-        attdocs = list()
         if matches:
             for link, sigdata in matches:
-                attdocs.append(LoreAttestationDocument(link, sigdata))
+                attdoc = LoreAttestationDocument(link, sigdata)
+                attdocs.append(attdoc)
 
         return attdocs
 
@@ -1061,24 +1169,27 @@ class LoreAttestation:
         self.passing = False
         self.attdocs = list()
 
-    def validate(self, lore_lookup=True):
+    def _check_if_passing(self):
         global ATTESTATIONS
-
         hg = (self.i, self.m, self.p)
         for attdoc in ATTESTATIONS:
             if hg in attdoc.hashes and attdoc.passing:
                 self.passing = True
                 self.attdocs.append(attdoc)
 
-        if len(self.attdocs) or not lore_lookup:
-            return
+    def validate(self, lore_lookup=True):
+        global ATTESTATIONS
+        self._check_if_passing()
 
-        attdocs = LoreAttestationDocument.get_from_lore(self.attid)
-        ATTESTATIONS += attdocs
-        for attdoc in attdocs:
-            if hg in attdoc.hashes and attdoc.passing:
-                self.passing = True
-                self.attdocs.append(attdoc)
+        if not len(self.attdocs):
+            attdocs = LoreAttestationDocument.get_from_cache(self.attid)
+            ATTESTATIONS += attdocs
+            self._check_if_passing()
+
+        if not len(self.attdocs) and lore_lookup:
+            attdocs = LoreAttestationDocument.get_from_lore(self.attid)
+            ATTESTATIONS += attdocs
+            self._check_if_passing()
 
     def __repr__(self):
         out = list()
@@ -1178,6 +1289,36 @@ def get_main_config():
     return MAIN_CONFIG
 
 
+def get_cache_dir():
+    global _CACHE_CLEANED
+    if 'XDG_CACHE_HOME' in os.environ:
+        cachehome = os.environ['XDG_CACHE_HOME']
+    else:
+        cachehome = os.path.join(str(Path.home()), '.cache')
+    cachedir = os.path.join(cachehome, 'b4')
+    Path(cachedir).mkdir(parents=True, exist_ok=True)
+    if _CACHE_CLEANED:
+        return cachedir
+
+    # Delete all .mbx and .lookup files older than cache-expire
+    config = get_main_config()
+    try:
+        expmin = int(config['cache-expire']) * 60
+    except ValueError:
+        logger.critical('ERROR: cache-expire must be an integer (minutes): %s', config['cache-expire'])
+        expmin = 600
+    expage = time.time() - expmin
+    for entry in os.listdir(cachedir):
+        if entry.find('.mbx') <= 0 and entry.find('.lookup') <= 0:
+            continue
+        st = os.stat(os.path.join(cachedir, entry))
+        if st.st_mtime < expage:
+            logger.debug('Cleaning up cache: %s', entry)
+            os.unlink(os.path.join(cachedir, entry))
+    _CACHE_CLEANED = True
+    return cachedir
+
+
 def get_user_config():
     global USER_CONFIG
     if USER_CONFIG is None:
@@ -1189,7 +1330,7 @@ def get_requests_session():
     global REQSESSION
     if REQSESSION is None:
         REQSESSION = requests.session()
-        REQSESSION.headers.update({'User-Agent': 'b4/%s' % VERSION})
+        REQSESSION.headers.update({'User-Agent': 'b4/%s' % __VERSION__})
     return REQSESSION
 
 
